/**
 * @author santa
 * @created Nov 14, 2008
 */
package search.maze;

import java.awt.Point;
import java.util.LinkedList;


public class BFSEngine extends SearchEngine {

    public BFSEngine(Maze m) {
        super(m);
    }

    protected Node doSearch() {
        LinkedList<Node> queue = new LinkedList<Node>();
        Node current = new Node(maze.getStart(), null);

        queue.offer(current);
        while (!queue.isEmpty()) {
            current = queue.poll();
            if (maze.getValue(current.ref.x, current.ref.y) == Maze.VISITED)
                continue;
            else
                maze.setValue(current.ref.x, current.ref.y, Maze.VISITED);

            if (current.ref.equals(maze.getGoal())) {
                return current;
            } else {
                Point[] moves = maze.getPossibleMoves(current.ref);
                for (Point move : moves) {
                    Node moveNode = new Node(move, current);
                    queue.offer(moveNode);
                }
            }

            try {
                Thread.sleep(delay);
            } catch (InterruptedException e) {
            }
        }

        return null;
    }

    // for unit test
    public static void main(String[] args) {
        SearchEngine bfs = new BFSEngine(new Maze(10, 10));
        System.out.println("Generating a maze ...");
        System.out.println(bfs.getMaze());
        System.out.print("Searching for a path ... ");
        try {
            bfs.search();
        } catch (PathNotFoundException e) {
            System.out.println("Not found.");
            System.exit(0);
        }
        System.out.println("Done.");
        System.out.println(bfs.getMaze());
        System.out.println("Path:");

        Point[] path = bfs.getPath();
        StringBuffer sb = new StringBuffer("S -> ");
        for (int i = 0; i < path.length; i++)
            sb.append("(" + path[i].x + "," + path[i].y + ") -> ");
        sb.append("G");
        System.out.println(sb.toString());

        System.exit(0);
    }

}
